//+------------------------------------------------------------------+
//|                                                     T3 basic.mq4 |
//|                                                           mladen |
//|                                                                  |
//|                                                                  |
//| original T3 developed by Tim Tilson (TASC January 1998)          |
//| period lag modification by Bob Fulks and Alex Matulich 4/2003    |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_chart_window
#property indicator_buffers 5
#property indicator_color1  Gray
#property indicator_color2  LimeGreen
#property indicator_color3  LimeGreen
#property indicator_color4  PaleVioletRed
#property indicator_color5  PaleVioletRed
#property indicator_width2  2
#property indicator_width3  2
#property indicator_width4  2
#property indicator_width5  2

//
//
//
//
//

extern string TimeFrame  = "Current time frame";
extern int    T3Period   = 14;
extern int    T3Price    = PRICE_CLOSE;
extern double T3Hot      = 0.7;
extern bool   T3Original = false;
extern double SlopeLimit = 9999;

//
//
//
//
//

double t3[];
double t3Ua[];
double t3Ub[];
double t3Da[];
double t3Db[];
double slope[];
double emas[][6];
double alpha;
double c1;
double c2;
double c3;
double c4;

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,t3);
   SetIndexBuffer(1,t3Ua);
   SetIndexBuffer(2,t3Ub);
   SetIndexBuffer(3,t3Da);
   SetIndexBuffer(4,t3Db);
   SetIndexBuffer(5,slope);

   //
   //
   //
   //
   //
   
      double a  = T3Hot;
             c1 = -a*a*a;
             c2 = 3*a*a+3*a*a*a;
             c3 = -6*a*a-3*a-3*a*a*a;
             c4 = 1+3*a+a*a*a+3*a*a;
               
   //
   //
   //
   //
   //
               
      T3Period = MathMax(1,T3Period);
      if (T3Original)
           alpha = 2.0/(1.0 + T3Period);
      else alpha = 2.0/(2.0 + (T3Period-1.0)/2.0);

      //
      //
      //
      //
      //
            
         indicatorFileName = WindowExpertName();
         calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) { return(0); }
         returnBars        = TimeFrame=="returnBars";     if (returnBars)     { return(0); }
         timeFrame         = stringToTimeFrame(TimeFrame);
      
   IndicatorShortName(timeFrameToString(timeFrame)+" T3 ("+T3Period+")");
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { t3[0] = MathMin(limit+1,Bars-1); return(0); }

   //
   //
   //
   //
   //

   double pipMultiplier = 1; if (Digits==3 || Digits==5) pipMultiplier = 10;
   double diff = SlopeLimit*Point*pipMultiplier;
   if (calculateValue || timeFrame == Period())
   {
      if (ArrayRange(emas,0) != Bars) ArrayResize(emas,Bars);
      if (slope[limit]==-1) CleanPoint(limit,t3Da,t3Db);
      if (slope[limit]== 1) CleanPoint(limit,t3Ua,t3Ub);
         for(i=limit; i>=0; i--)
         {
            t3[i] = iT3(iMA(NULL,0,1,0,MODE_SMA,T3Price,i),i);
            t3Ua[i] = EMPTY_VALUE;
            t3Ub[i] = EMPTY_VALUE;
            t3Da[i] = EMPTY_VALUE;
            t3Db[i] = EMPTY_VALUE;
            slope[i] = slope[i+1];
               if (t3[i]>t3[i+1] && (t3[i]-t3[i+1])<diff) slope[i] =  1;
               if (t3[i]<t3[i+1] && (t3[i+1]-t3[i])<diff) slope[i] = -1;
               if (MathAbs(t3[i]-t3[i+1])>diff)           slope[i] =  0;
               if (slope[i]==-1) PlotPoint(i,t3Da,t3Db,t3);
               if (slope[i]== 1) PlotPoint(i,t3Ua,t3Ub,t3);
         }            
         return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   if (slope[limit]==-1) CleanPoint(limit,t3Da,t3Db);
   if (slope[limit]== 1) CleanPoint(limit,t3Ua,t3Ub);
   for (i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         t3[i]    = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",T3Period,T3Price,T3Hot,T3Original,SlopeLimit,0,y);
         slope[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",T3Period,T3Price,T3Hot,T3Original,SlopeLimit,5,y);
         t3Ua[i]  = EMPTY_VALUE;
         t3Ub[i]  = EMPTY_VALUE;
         t3Da[i]  = EMPTY_VALUE;
         t3Db[i]  = EMPTY_VALUE;
               if (slope[i]==-1) PlotPoint(i,t3Da,t3Db,t3);
               if (slope[i]== 1) PlotPoint(i,t3Ua,t3Ub,t3);
   }
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double iT3(double price,int shift)
{
   int i = Bars-shift-1;
   if (i < 1)
      {
         emas[i][0] = price;
         emas[i][1] = price;
         emas[i][2] = price;
         emas[i][3] = price;
         emas[i][4] = price;
         emas[i][5] = price;
      }
   else
      {
         emas[i][0] = emas[i-1][0]+alpha*(price     -emas[i-1][0]);
         emas[i][1] = emas[i-1][1]+alpha*(emas[i][0]-emas[i-1][1]);
         emas[i][2] = emas[i-1][2]+alpha*(emas[i][1]-emas[i-1][2]);
         emas[i][3] = emas[i-1][3]+alpha*(emas[i][2]-emas[i-1][3]);
         emas[i][4] = emas[i-1][4]+alpha*(emas[i][3]-emas[i-1][4]);
         emas[i][5] = emas[i-1][5]+alpha*(emas[i][4]-emas[i-1][5]);
      }
   return(c1*emas[i][5] + c2*emas[i][4] + c3*emas[i][3] + c4*emas[i][2]);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}